
;--------------------------------------------------------;
; File mainwin.asm                                       ;
; Applicazione Win32 standard dotata di Main Window      ;
;--------------------------------------------------------;

; ################# direttive per l'assembler #################

.386                                   ; set di istruzioni a 32 bit
.MODEL      FLAT, STDCALL              ; memory model & calling conventions
OPTION      CASEMAP: NONE              ; case sensitive on symbols

; ############### dichiarazione tipi e costanti ###############

; ########################## macro ############################

; trasferimento dati da mem32 a mem32

MOV32 MACRO Destinazione, Sorgente

   push     dword ptr Sorgente
   pop      dword ptr Destinazione

ENDM

; #################### inclusione librerie ####################

INCLUDE     ..\include\win32.inc       ; header file principale di Win32
INCLUDE     ..\include\user32.inc      ; interfaccia per USER32.LIB
INCLUDE     ..\include\kernel32.inc    ; interfaccia per KERNEL32.LIB

INCLUDELIB  ..\lib\import32.lib        ; libreria generale

; ################# prototipi delle procedure #################

WinMain     PROTO    :DWORD, :DWORD, :DWORD, :DWORD

; ################ segmento dati inizializzati #################

_DATA       SEGMENT  DWORD PUBLIC USE32 'DATA'

className   db       'MainWin', 0
winTitle    db       'Win32 Assembly - Applicazione MainWin', 0

msgTesto    db       'Chiudere MAINWIN.EXE ?', 0
msgTitolo   db       'Win 32 Assembly', 0

_DATA       ENDS

; ############## segmento dati non inizializzati ##############

_BSS        SEGMENT  DWORD PUBLIC USE32 'BSS'

hInstance   dd    ?                    ; handle dell'applicazione
commandLine dd    ?                    ; puntatore alla command line
hWindow     dd    ?                    ; handle della main window

_BSS        ENDS

; ##################### segmento di codice #####################

_TEXT       SEGMENT DWORD PUBLIC USE32 'CODE'

start:                                 ; entry point del programma

; ----------------- blocco inizializzazioni --------------------

   call     GetModuleHandleA, NULL     ; trova l'handle del modulo MAINWIN.EXE
   mov      hInstance, eax             ; e lo salva in hInstance

   call     GetCommandLineA            ; trova l'indirizzo della command line
   mov      commandLine, eax           ; e lo salva in commandLine

; ------------ blocco definizione della main window ------------

   call     WinMain, hInstance, NULL, commandLine, SW_SHOWNORMAL

; ---------------- blocco deinizializzazioni -------------------

   call     ExitProcess, eax           ; exit code restituito da WinMain

;----------------------------------------------------------------------------------;
; int WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpCmdLine, int nCmdShow) ;
;----------------------------------------------------------------------------------;
; Procedura principale dell'applicazione MAINWIN.ASM. All'interno di WinMain si    ;
; svolgono le fasi di inizializzazione, registrazione, attivazione e gestione      ;
; della Main Window.                                                               ;
;----------------------------------------------------------------------------------;

WinMain proc hInst :DWORD, hPrevInst :DWORD, lpCmdLine :DWORD, nCmdShow :DWORD

   LOCAL    wc       :WNDCLASSEX       ; crea wc nello stack
   LOCAL    msg      :MSG              ; crea msg nello stack

; inizializzazione main window

   mov      wc.wc_cbSize, SIZE WNDCLASSEX          ; dimensioni struttura
   mov      wc.wc_style, CS_HREDRAW OR CS_VREDRAW  ; stile della win. class
   mov      wc.wc_lpfnWndProc, offset WndProc      ; indirizzo window procedure
   mov      wc.wc_cbClsExtra, 0                    ; byte extra per la win. class
   mov      wc.wc_cbWndExtra, 0                    ; byte extra per la win. instance
   MOV32    wc.wc_hInstance, hInstance             ; handle dell'applicazione
   call     LoadIconA, NULL, IDI_WINLOGO           ; eax = handle icona
   mov      wc.wc_hIcon, eax                       ; icona dell'applicazione
   call     LoadCursorA, NULL, IDC_ARROW           ; eax = handle cursore
   mov      wc.wc_hCursor, eax                     ; cursore dell'applicazione
   mov      wc.wc_hbrBackground, COLOR_WINDOW + 1  ; colore di sfondo della finestra
   mov      wc.wc_lpszClassName, offset className  ; nome della classe finestra
   mov      wc.wc_hIconSm, NULL                    ; icona piccola dell'applicazione

; registrazione main window

   lea      eax, wc
   call     RegisterClassExA, eax                  ; richiesta di registrazione
   test     eax, eax                               ; errore ?
   jz       exitWinMain                            ; termina con eax = 0

; attivazione della main window

   call     CreateWindowExA, WS_EX_OVERLAPPEDWINDOW, \   ; stile esteso finestra
                             offset className, \         ; nome window class
                             offset winTitle, \          ; titolo finestra
                             WS_OVERLAPPEDWINDOW, \      ; stile ordinario finestra
                             CW_USEDEFAULT, \            ; x vertice superiore sinistro
                             CW_USEDEFAULT, \            ; y vertice superiore sinistro
                             CW_USEDEFAULT, \            ; larghezza finestra
                             CW_USEDEFAULT, \            ; altezza finestra
                             NULL, \                     ; handle finestra madre
                             NULL, \                     ; codice menu
                             hInstance, \                ; handle applicazione
                             NULL                        ; puntatore a CREATESTRUCT

   test     eax, eax                               ; errore ?
   jz       exitWinMain                            ; termina con eax = 0
   mov      hWindow, eax                           ; salva l'HWND della finestra in hWindow
   call     ShowWindow, hWindow, nCmdShow          ; visualizza la finestra
   call     UpdateWindow, hWindow                  ; aggiorna l'area client
   test     eax, eax                               ; errore ?
   jz       exitWinMain                            ; termina con eax = 0

; gestione della main window

MessageLoop:
   lea      eax, msg   
   call     GetMessage, eax, NULL, 0, 0            ; estrazione prossimo messaggio
   test     eax, eax                               ; eax = 0 ?
   jz       exitMessageLoop                        ; fine del loop (WM_QUIT)
   lea      eax, msg
   call     TranslateMessage, eax                  ; decodifica messaggio
   lea      eax, msg
   call     DispatchMessage,  eax                  ; invio messaggio alla WinProc
   jmp      MessageLoop                            ; ripeti
exitMessageLoop:

   mov      eax, msg.ms_wParam                     ; exit code = OK

exitWinMain:                                       ; exit code = 0 (errore)

   ret

WinMain endp

;----------------------------------------------------------------------------------;
; LRESULT WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)              ;
;----------------------------------------------------------------------------------;
; Window procedure della Main Window dell'applicazione MAINWIN.ASM. WndProc viene  ;
; utilizzata dal SO per inviare i messaggi alla Main Window.                       ;
;----------------------------------------------------------------------------------;

WndProc proc hWnd :DWORD, uMsg :DWORD, wParam :DWORD, lParam :DWORD

messaggio_WM_CLOSE:
   cmp      uMsg, WM_CLOSE             ; (uMsg == WM_CLOSE) ?
   jne      messaggio_WM_DESTROY
   call     MessageBoxA, hWnd, offset msgTesto, offset msgTitolo, MB_YESNO OR MB_ICONQUESTION
   cmp      eax, IDYES
   jne      exitWndProc
   jmp      messaggio_restituito       ; passa WM_CLOSE a DefWindowProc
messaggio_WM_DESTROY:
   cmp      uMsg, WM_DESTROY           ; (uMsg == WM_DESTROY) ?
   jne      messaggio_restituito
   call     PostQuitMessage, NULL
   jmp      exitWndProc
messaggio_restituito:
   call     DefWindowProc, hWnd, uMsg, wParam, lParam
   ret
exitWndProc:

   xor      eax, eax
   ret

WndProc endp

_TEXT       ENDS

; ##############################################################

   END      start                   ; fine del modulo
